/**
  */

/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "hw_config.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/

static void SystemClock_Config(void);
static void Work_All_Initialization(void);
static void Work_Event_Process(void);

/* Private functions ---------------------------------------------------------*/

static void Work_All_Initialization(void)
{
    /* STM32F4xx HAL library initialization:
       - Configure the Flash prefetch, instruction and Data caches
       - Configure the Systick to generate an interrupt each 1 msec
       - Set NVIC Group Priority to 4
       - Global MSP (MCU Support Package) initialization
     */
    HAL_Init();

    /* Configure the system clock */
    SystemClock_Config();

    /* Initialize Debug UART */
    USART_Debug_Initialization();

    /* Initialize LEDs */
    LED_Initialization();

    /* Initialize Keys */
    KEY_Initialization();

    /* Initialize SRAM */
    SRAM_Initialization();
}

static void Work_Event_Process(void)
{
    EventQueue_Data_S eventData;

    while(CRZ_FALSE == EventQueue_Is_Empty())
    {
        eventData = EventQueue_DeQueue();

        switch(eventData.event_id)
        {
        /*
         * Work Events
         */

        case CRZ_EVENT_WORK_USER_KEY_1_PRESS:
            MANGO_DBG_BLUE("USER Key 1 pressed\n");
            break;

        case CRZ_EVENT_WORK_USER_KEY_1_RELEASE:
            MANGO_DBG_BLUE("USER Key 1 released\n");
            break;

        case CRZ_EVENT_WORK_USER_KEY_2_PRESS:
            MANGO_DBG_BLUE("USER Key 2 pressed\n");
            break;

        case CRZ_EVENT_WORK_USER_KEY_2_RELEASE:
            MANGO_DBG_BLUE("USER Key 2 released\n");
            break;

        /*
         * Error Events
         */

        default:
            MANGO_ERROR("Not support Event: %d\n", eventData.event_data);
            break;
        }
    }
}

static CRZ_bool g_print_menu_control_flag = CRZ_TRUE;

static void print_menu_command(void)
{
    if(CRZ_TRUE == g_print_menu_control_flag)
    {
        printf("\n---------------------\n");
        printf("Press menu key\n");
        printf("---------------------\n");
        printf("1> LED Test\n");
        printf("2> SDMMC Test\n");
        printf("3> SRAM Test\n");
        printf("4> USB Host Test\n");
        printf("5> Ethernet HTTP Test\n");
        printf("6> USB Device Test\n");
        printf("7> RTC Calendar Test\n");
        printf("8> CAN Test\n");
        printf("9> UART6 Test\n");
        printf("---------------------\n");
    }

    g_print_menu_control_flag = CRZ_FALSE;
}

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void)
{
    char ch;

    Work_All_Initialization();

    MANGO_DBG("Mango-M32F4, Date: %s, Time: %s\n", __DATE__, __TIME__);

    HAL_Delay(100);
    Wait_N_Seconds(1);

    /* Infinite loop */
    while (1)
    {
        /* Process Event */
        if(CRZ_FALSE == EventQueue_Is_Empty())
        {
            Work_Event_Process();
        }

        print_menu_command();

        if(CRZ_TRUE == UartQueue_Serial_Is_Empty())
        {
            continue;
        }

        ch = UartQueue_Serial_DeQueue();

        if(0 != ch)
        {
            g_print_menu_control_flag = CRZ_TRUE;

            MANGO_DBG_BLUE("%c is selected\n", ch);

            switch(ch)
            {
            case '1':
                LED_Test();
                break;

            case '2':
                SDMMC_Test();
                break;

            case '3':
                SRAM_Test();
                break;

            case '4':
                USB_Host_Test();
                break;

            case '5':
                Ethernet_HTTP_Test();
                break;

            case '6':
                USB_Device_Test();
                break;

            case '7':
                RTC_Test();
                break;

            case '8':
                CAN_Test();
                break;

            case '9':
                UART6_Test();
                break;
            }
        }
    }
}

/**
  * @brief  System Clock Configuration
  *         The system Clock is configured as follow : 
  *            System Clock source            = PLL (HSE)
  *            SYSCLK(Hz)                     = 168000000
  *            HCLK(Hz)                       = 168000000
  *            AHB Prescaler                  = 1
  *            APB1 Prescaler                 = 4
  *            APB2 Prescaler                 = 2
  *            HSE Frequency(Hz)              = 25000000
  *            PLL_M                          = 25
  *            PLL_N                          = 336
  *            PLL_P                          = 2
  *            PLL_Q                          = 7
  *            VDD(V)                         = 3.3
  *            Main regulator output voltage  = Scale1 mode
  *            Flash Latency(WS)              = 5
  * @param  None
  * @retval None
  */
static void SystemClock_Config(void)
{
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_OscInitTypeDef RCC_OscInitStruct;

  /* Enable Power Control clock */
  __HAL_RCC_PWR_CLK_ENABLE();

  /* The voltage scaling allows optimizing the power consumption when the device is 
     clocked below the maximum system frequency, to update the voltage scaling value 
     regarding system frequency refer to product datasheet.  */
  __HAL_PWR_VOLTAGESCALING_CONFIG(PWR_REGULATOR_VOLTAGE_SCALE1);

  /* Enable HSE Oscillator and activate PLL with HSE as source */
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSE;
  RCC_OscInitStruct.HSEState = RCC_HSE_ON;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_ON;
  RCC_OscInitStruct.PLL.PLLSource = RCC_PLLSOURCE_HSE;
  RCC_OscInitStruct.PLL.PLLM = 25;
  RCC_OscInitStruct.PLL.PLLN = 336;
  RCC_OscInitStruct.PLL.PLLP = RCC_PLLP_DIV2;
  RCC_OscInitStruct.PLL.PLLQ = 7;
  HAL_RCC_OscConfig(&RCC_OscInitStruct);
  
  /* Select PLL as system clock source and configure the HCLK, PCLK1 and PCLK2 
     clocks dividers */
  RCC_ClkInitStruct.ClockType = (RCC_CLOCKTYPE_SYSCLK | RCC_CLOCKTYPE_HCLK | RCC_CLOCKTYPE_PCLK1 | RCC_CLOCKTYPE_PCLK2);
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_PLLCLK;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV4;  
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV2;  
  HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_5);

  /* STM32F405x/407x/415x/417x Revision Z devices: prefetch is supported  */
  if (HAL_GetREVID() == 0x1001)
  {
    /* Enable the Flash prefetch */
    __HAL_FLASH_PREFETCH_BUFFER_ENABLE();
  }
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

  /* Infinite loop */
  while (1)
  {
  }
}
#endif

/**
  * @}
  */ 

/**
  * @}
  */ 

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
